﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Windows.Forms;
using System.Drawing;
using System.Runtime.InteropServices;

using SBSDKComWrapperLib;

using System.Threading;

namespace ttBarebone
{
    public class Controller
    {
        //Ratio between Smartboard Tracking and Screen
        public const float SBSDK_NON_PROJECTED_RANGE = 4096;

        private bool fullscreen;
        private View view;

        private FormWindowState windowState;
        private FormBorderStyle borderStyle;
        private bool topMost;
        private Rectangle bounds;

        //Smartboard stuff
        private bool smartboard;
        private SBSDKBaseClassClass sbsdk;
        private _ISBSDKBaseClassEvents_Event sbsdkEvents;


        public Controller(View view)
        {
            this.fullscreen = false;
            this.smartboard = false;

            this.view = view;
        }

        #region Keyboard Event handlers
        public void KeyDown(object o, System.Windows.Forms.KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case System.Windows.Forms.Keys.F:
                    ToggleFullscreen();
                    break;
                case System.Windows.Forms.Keys.S:
                    if (!this.smartboard && InitSmartboard())
                    {
                        this.smartboard = true;
                        view.Console("Smartboard active!");
                    }
                    else
                    {
                        view.Console("Error initialising Smartboard!");
                    }
                    break;
                case System.Windows.Forms.Keys.Escape:
                    QuitApplication();
                    break;
            }
        }

        public void KeyUp(object o, System.Windows.Forms.KeyEventArgs e)
        {
        }
        #endregion

        #region Smartboard Event Handlers 
        public void SmartboardDown(int x, int y, int z, int id)
        {
            Point actPos = GetScreenPointFromBoardPoint(x, y);
            Down(actPos.X, actPos.Y, id);
            //view.Console("down input #" + id + " at (" + actPos.X + ", " + actPos.Y + ")");
        }

        public void SmartboardUp(int x, int y, int z, int id)
        {
            Point actPos = GetScreenPointFromBoardPoint(x, y);
            Up(actPos.X, actPos.Y, id);
            //view.Console("up input #" + id + " at (" + actPos.X + ", " + actPos.Y + ")");
        }

        public void SmartboardMove(int x, int y, int z, int id)
        {
            Point actPos = GetScreenPointFromBoardPoint(x, y);
            Move(actPos.X, actPos.Y, id);
            //view.Console("move input #" + id + " at (" + actPos.X + ", " + actPos.Y + ")");
        }
        #endregion

        /// <summary>
        /// A down event was reported either by a mouse or tabletop
        /// event handler.
        /// Either a mouse button was pressed or a finger tapped at
        /// the coordinates (x, y).
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="id"></param>
        public void Down(int x, int y, int id)
        {
            // Handling of this event:
            // ...

            view.SetInput(id, x, y);
        }

        /// <summary>
        /// A move event was reported either by a mouse or tabletop
        /// event handler.
        /// Either a mouse or finger moved to the coordinates (x, y).
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="id"></param>
        public void Move(int x, int y, int id)
        {
            // Handling of this event:
            // ...

            view.SetInput(id, x, y);
        }

        /// <summary>
        /// An up event was reported either by a mouse or tabletop
        /// event handler.
        /// Either a mouse button was released or a finger left the 
        /// surface at the coordinates (x, y).
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="id"></param>
        public void Up(int x, int y, int id)
        {
            // Handling of this event:
            // ...

            view.ReleaseInput(id);
        }

        private bool InitSmartboard()
        {
            try
            {
                this.sbsdk = new SBSDKBaseClassClass();

                this.sbsdkEvents = (_ISBSDKBaseClassEvents_Event)sbsdk;

                //attach smartboard eventlisteners
                this.sbsdkEvents.OnXYNonProjectedDown += new _ISBSDKBaseClassEvents_OnXYNonProjectedDownEventHandler(this.SmartboardDown);
                this.sbsdkEvents.OnXYNonProjectedMove += new _ISBSDKBaseClassEvents_OnXYNonProjectedMoveEventHandler(this.SmartboardMove);
                this.sbsdkEvents.OnXYNonProjectedUp += new _ISBSDKBaseClassEvents_OnXYNonProjectedUpEventHandler(this.SmartboardUp);

                if (!sbsdk.SBSDKIsABoardConnected())
                {
                    Console.WriteLine("board not connected!");
                }
            }
            catch
            {
                return false;
            }
            return true;
        }


        /// <summary>
        /// Converts a point from smartboard to screen coordinates.
        /// </summary>
        /// <param name="x">Horizontal position of the point in Smartboard coordinates</param>
        /// <param name="y">Vertical position of the point in Smartboard coordinates</param>
        /// <returns></returns>
        private Point GetScreenPointFromBoardPoint(int x, int y)
        {
            Point point = new Point(x, y);

            System.Drawing.Rectangle screen = System.Windows.Forms.Screen.PrimaryScreen.Bounds;

            // Make new x,y coordinates by mapping the board coordinates onto the view

            float factorW = (float)screen.Width / SBSDK_NON_PROJECTED_RANGE;
            point.X = (int)(point.X * factorW);


            float factorH = (float)screen.Height / SBSDK_NON_PROJECTED_RANGE;
            point.Y = (int)(point.Y * factorH);
            return point;
        }

        /// <summary>
        /// Switches the display between Fullscreen and Window mode.
        /// </summary>
        private void ToggleFullscreen()
        {
            if (this.fullscreen)
            {
                view.WindowState = this.windowState;
                view.FormBorderStyle = this.borderStyle;
                view.TopMost = this.topMost;
                view.Bounds = this.bounds;

                this.fullscreen = false;
            }
            else
            {
                SaveForm();

                view.WindowState = FormWindowState.Maximized;
                view.FormBorderStyle = FormBorderStyle.None;
                view.TopMost = true;

                WinApi.SetWinFullScreen(view.Handle);

                this.fullscreen = true;
            }
        }

        private void SaveForm()
        {
            this.windowState = view.WindowState;
            this.borderStyle = view.FormBorderStyle;
            this.topMost = view.TopMost;
            this.bounds = view.Bounds;
        }


        private void QuitApplication()
        {
            System.Environment.Exit(0);
        }

        /// <summary>
        /// Accesses the "user32"-DLL file to allow for an easy switch between
        /// fullscreen and window mode. Just ignore it and hope it works ;)
        /// </summary>
        private class WinApi
        {
            [DllImport("user32.dll", EntryPoint = "GetSystemMetrics")]
            public static extern int GetSystemMetrics(int which);

            [DllImport("user32.dll")]
            public static extern void SetWindowPos(IntPtr hwnd, IntPtr hwndInsertAfter, int X, int Y, int width, int height, uint flags);

            private const int SM_CXSCREEN = 0;
            private const int SM_CYSCREEN = 1;
            private static IntPtr HWND_TOP = IntPtr.Zero;
            private const int SWP_SHOWWINDOW = 64;

            public static int ScreenX
            {
                get { return GetSystemMetrics(SM_CXSCREEN); }
            }

            public static int ScreenY
            {
                get { return GetSystemMetrics(SM_CYSCREEN); }
            }

            public static void SetWinFullScreen(IntPtr hwnd)
            {
                SetWindowPos(hwnd, HWND_TOP, 0, 0, ScreenX, ScreenY, SWP_SHOWWINDOW);
            }
        }
    }
}
